@@ -691,6 +691,8 @@ class RedisCluster(Redis):
                 log.exception("AskError")
 
                 redirect_addr, asking = "{0}:{1}".format(e.host, e.port), True
+            except Exception:
+                log.exception("OtherException")
             finally:
                 self.connection_pool.release(connection)
 
